<?php

/**
 * @file
 * CollectiveAccess Feeds specific implementation of taxonomy target
 *
 * Adds:
 * - support for adding new terms in non-tags vocabularies
 * - support to remove earlier assigned terms upon item update
 * - better language term support
 */

function _collectiveaccess_feeds_feeds_node_processor_targets_taxonomy_alter(&$targets, $content_type) {
  foreach (taxonomy_get_vocabularies($content_type) as $vocabulary) {
    $description = t('This will map CollectiveAccess data intelligently to the @name vocabulary', array('@name' => $vocabulary->name));

    module_load_include('inc', 'feeds', 'mappers/taxonomy');
    $targets['collectiveaccess_feeds_taxonomy:'. taxonomy_vocabulary_id($vocabulary)] = array(
      'name' => t('Taxonomy: @name (for CollectiveAccess)', array('@name' => $vocabulary->name)),
      'callback' => 'collectiveaccess_feeds_taxonomy_set_target',
      'description' => check_plain($description),
      'real_target' => 'taxonomy',
    );
  }
}

/**
 * Callback for mapping. Here is where the actual mapping happens.
 *
 * @param $node
 *   Reference to the node object we are working on.
 *
 * @param $key
 *   A key as added to the $targets array by
 *   _collectiveaccess_feeds_feeds_node_processor_targets_alter().
 *
 * @param $terms
 *   Given terms as array.
 *
 * Add the given terms to the node object so the taxonomy module can add them
 * on node_save().
 *
 * Based on taxonomy_feeds_set_target in feeds/mappers/taxonomy.inc
 */
function collectiveaccess_feeds_taxonomy_set_target(&$node, $key, $terms) {

  // Return if there are no terms.
  if (empty($terms)) {
    return;
  }

  // Load target vocabulary to check, if it has the "tags" flag.
  $vocabulary = taxonomy_get_vocabulary(str_replace('collectiveaccess_feeds_taxonomy:', '', $key));
  if (function_exists('i18ntaxonomy_vocabulary')) {
    $translation_mode = i18ntaxonomy_vocabulary($vocabulary->vid);
  }
  else {
    $translation_mode = FALSE;
  }

  // Remove old terms from node if this is an update operation
  if (variable_get('collectiveaccess_feeds_taxonomy_cleanup_on_update', TRUE) == TRUE) {
    if (isset($node->taxonomy) && is_array($node->taxonomy)) {
      foreach ($node->taxonomy as $tid => $term) {
        if ($term->vid == $vocabulary->vid) {
          unset($node->taxonomy[$tid]);
        }
      }
    }
  }

  // Cast a given single string to an array so we can use it.
  if (!is_array($terms)) {
    $terms = array($terms);
  }

  if ($vocabulary->tags) {
    foreach ($terms as $k => $v) {
      // Make sure there aren't any terms with a comma (=tag delimiter) in it.
      $terms[$k] = preg_replace('/\s*,\s*/', ' ', $v);
    }
    // Simply add a comma separated list to the node for a "tags" vocabulary.
    $terms = array_merge($terms, drupal_explode_tags($node->taxonomy['tags'][$vocabulary->vid]));
    $node->taxonomy['tags'][$vocabulary->vid] = implode(',', $terms);
  }
  else {
    foreach ($terms as $term) {
      if ($term instanceof FeedsTermElement) {
        $node->taxonomy[$term->tid] = (object)$term;
      }
      // Check if a term already exists.
      // Inspired by http://drupal.org/node/714424#comment-4046948
      if ($translation_mode && $translation_mode == I18N_TAXONOMY_TRANSLATE) {
        $language = $node->language;
      }
      else {
        $language = '';
      }
      $terms_found = collectiveaccess_feeds_taxonomy_get_term_by_name_vid($term, $vocabulary->vid, $language);
      if (empty($terms_found) && !empty($term)) {
        // We should add taxonomy term
        $new_term = array(
          'name' => $term,
          'vid' => $vocabulary->vid,
          'language' => $node->language,
        );
        $status = taxonomy_save_term($new_term);
        // Check to see if everything went right
        if ($status == SAVED_NEW) {
          $terms_found = array((object) $new_term);
        }
        else {
          break;
        }
      }

      // If any terms are found add them to the node's taxonomy by found tid.
      foreach ($terms_found AS $term_found) {
        $node->taxonomy[$term_found->tid] = $term_found;
        if (!$vocabulary->multiple) {
          break;
        }
      }
      // If the vocab is not for multiple tags break after the first hit.
      if (!$vocabulary->multiple) {
        break;
      }
    }
  }
}

/**
 * Try to map a string to an existing term by name and vocabulary id.
 *
 * Provides a case-insensitive and trimmed mapping, to maximize the likelihood
 * of a successful match limited by a vocabulary id.
 *
 * @param $name
 *   Name of the term to search for.
 *
 * @param $vid
 *   The vocabulary's ID.
 *
 * @return
 *   An array of matching term objects.
 */
function collectiveaccess_feeds_taxonomy_get_term_by_name_vid($name, $vid, $language = NULL) {
  if ($language) {
    $db_result = db_query("SELECT t.tid, t.name FROM {term_data} t WHERE LOWER(t.name) = LOWER('%s') AND t.vid = %d AND t.language = '%s'", trim($name), $vid, $language);
  }
  else {
    $db_result = db_query("SELECT t.tid, t.name FROM {term_data} t WHERE LOWER(t.name) = LOWER('%s') AND t.vid = %d", trim($name), $vid);
  }
  $result = array();
  while ($term = db_fetch_object($db_result)) {
    $result[] = $term;
  }
  return $result;
}